#include "TreeWidget.h"

//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
TreeWidget::TreeWidget() :
    m_bDragStarted(false),
    m_pDragItem(nullptr)
{
  setDragEnabled(true);
  setAcceptDrops(true);
  setDragDropMode(QAbstractItemView::DragDrop);

  m_listMimeTypes << "MyType";
}

//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
void
TreeWidget::commitData(QWidget*)
{
	// Do not call the base class - ignore the change so that the QTreeWidget
	// will not change the item's label.
}

//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
QStringList
TreeWidget::mimeTypes() const
{
	return m_listMimeTypes;
}

//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
QMimeData*
TreeWidget::mimeData(const QList<QTreeWidgetItem*> items) const
{
    // Remember the source drag item when we start a drag ooperation.
    if ( m_bDragStarted )
    {
        QList<QTreeWidgetItem*> list = selectedItems();

        m_pDragItem = nullptr;
        if ( list.first() != nullptr )
	        m_pDragItem = list.first();
    }

    return QTreeWidget::mimeData(items);
}

//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
bool
TreeWidget::dropMimeData(QTreeWidgetItem*, int, const QMimeData*, Qt::DropAction)
{
	return false;
}

//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
Qt::DropActions
TreeWidget::supportedDropActions() const
{
	return Qt::CopyAction | Qt::MoveAction;
}

//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
void
TreeWidget::startDrag(Qt::DropActions supportedActions)
{
	m_bDragStarted = true;
	QTreeWidget::startDrag(supportedActions);
	m_bDragStarted = false;
}

//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
void
TreeWidget::dragEnterEvent(QDragEnterEvent* pEvent)
{
	QTreeWidget::dragEnterEvent(pEvent);
	pEvent->accept();
}

//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
void
TreeWidget::dragMoveEvent(QDragMoveEvent* pEvent)
{
	QTreeWidget::dragMoveEvent(pEvent);
	pEvent->accept();
}

//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
void
TreeWidget::dragLeaveEvent(QDragLeaveEvent* pEvent)
{
	QTreeWidget::dragLeaveEvent(pEvent);
	pEvent->accept();
}

//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
void
TreeWidget::dropEvent(QDropEvent* pEvent)
{
    QTreeWidgetItem* pTarget = itemAt(pEvent->pos().x(), pEvent->pos().y());
    if ( pTarget != nullptr )
    {
      QTreeWidgetItem* pNewItem = new QTreeWidgetItem;
        pNewItem->setText(0, m_pDragItem->text(0));

        if ( pTarget->parent() == nullptr )
        {
            addTopLevelItem(pNewItem);
        }
        else
        {
            pTarget->addChild(pNewItem);
        }

        if ( pEvent->dropAction() == Qt::MoveAction )
        {
            QTreeWidgetItem* pSourceParent = m_pDragItem->parent();
            QTreeWidgetItem* pTaken = nullptr;

            if ( pSourceParent == nullptr )
            {
                int iIndex = indexOfTopLevelItem(m_pDragItem);
                pTaken = takeTopLevelItem(iIndex);
            }
            else
            {
            int iIndex = pSourceParent->indexOfChild(m_pDragItem);
                pTaken = pSourceParent->takeChild(iIndex);
            }

            delete pTaken;
        }
    }

    // Unconditionally "accept" the drop to prevent QTreeWidget::dropEvent()
    // from processing the event and moving tree items around.  Also, set the
    // drop action to "ignore" so that QTreeWidget::startDrag() will not
    // clearOrRemove() tree items when the drop action is "move".

    pEvent->setDropAction(Qt::IgnoreAction);
    pEvent->accept();
    // Call the base class *after* we use 'pEvent' since QTreeWidget will modify
    // the event.
    //QTreeWidget::dropEvent(pEvent);
}
